@@ -1,3 +1,8 @@
+use log::{debug, trace};
+use rustc_middle::mir::*;
+use rustc_mir_dataflow::ResultsVisitor;
+use rustc_span::Span;
+
 use super::{
   analysis::{FlowAnalysis, FlowDomain},
   FlowResults,
@@ -8,13 +13,9 @@ use crate::{
     impls::{LocationSet, PlaceIndex, PlaceSet},
     IndexedDomain,
   },
-  mir::utils::PlaceExt,
-  source_map::{location_to_spans, HirSpanner},
+  mir::utils::SpanExt,
+  source_map::{EnclosingHirSpans, Spanner},
 };
-use log::{debug, trace};
-use rustc_middle::mir::*;
-use rustc_mir_dataflow::ResultsVisitor;
-use rustc_span::Span;
 
 #[derive(Clone, Copy, Debug)]
 pub enum Direction {
@@ -35,35 +36,33 @@ impl DepVisitor<'_, '_, 'tcx> {
     state: &FlowDomain<'tcx>,
     opt_location: Option<Location>,
     to_check: Vec<PlaceIndex>,
+    is_switch: bool,
   ) {
     for (target_locs, (out_locs, out_places)) in
       self.target_deps.iter().zip(self.outputs.iter_mut())
     {
-      for place in to_check.iter() {
-        if let Some(loc_deps) = state.row_set(*place) {
-          if loc_deps.indices().next().is_none() {
-            continue;
-          }
+      for (place, loc_deps) in to_check
+        .iter()
+        .map(|place| (place, state.row_set(*place)))
+        .filter(|(_, loc_deps)| !loc_deps.is_empty())
+      {
+        let matches = match self.direction {
+          Direction::Forward => loc_deps.is_superset(target_locs),
+          Direction::Backward => target_locs.is_superset(&loc_deps),
+        };
 
-          let matches = match self.direction {
-            Direction::Forward => loc_deps.is_superset(target_locs),
-            Direction::Backward => target_locs.is_superset(&loc_deps),
-          };
+        if matches {
+          trace!(
+            "{opt_location:?}: place {:?} (deps {loc_deps:?}) / target_locs {target_locs:?}",
+            state.row_domain.value(*place)
+          );
+          out_places.insert(*place);
 
-          if matches {
-            trace!(
-              "{:?}: place {:?} (deps {:?}) / target_locs {:?}",
-              opt_location,
-              state.row_domain.value(*place),
-              loc_deps,
-              target_locs
-            );
-            out_places.insert(*place);
-
-            if let Some(location) = opt_location {
-              if loc_deps.contains(location) {
-                out_locs.insert(location);
-              }
+          if let Some(location) = opt_location {
+            if loc_deps.contains(location)
+              || (is_switch && target_locs.contains(location))
+            {
+              out_locs.insert(location);
             }
           }
         }
@@ -83,7 +82,12 @@ impl ResultsVisitor<'mir, 'tcx> for DepVisitor<'_, 'mir, 'tcx> {
   ) {
     if block == START_BLOCK {
       let place_domain = self.analysis.place_domain();
-      self.visit(state, None, place_domain.all_args(self.analysis.body));
+      self.visit(
+        state,
+        None,
+        place_domain.all_args(self.analysis.body),
+        false,
+      );
     }
   }
 
@@ -99,6 +103,7 @@ impl ResultsVisitor<'mir, 'tcx> for DepVisitor<'_, 'mir, 'tcx> {
           state,
           Some(location),
           self.analysis.aliases.conflicts(lhs).indices().collect(),
+          false,
         );
       }
       _ => {}
@@ -123,6 +128,7 @@ impl ResultsVisitor<'mir, 'tcx> for DepVisitor<'_, 'mir, 'tcx> {
         state,
         Some(location),
         self.analysis.place_domain().as_vec().indices().collect(),
+        matches!(terminator.kind, TerminatorKind::SwitchInt { .. }),
       );
     }
   }
@@ -138,33 +144,18 @@ pub fn compute_dependencies(
   let body = results.analysis.body;
   let aliases = &results.analysis.aliases;
 
-  let new_location_set = || LocationSet::new(results.analysis.location_domain().clone());
-  let new_place_set = || PlaceSet::new(results.analysis.place_domain().clone());
+  let new_location_set = || LocationSet::new(results.analysis.location_domain());
+  let new_place_set = || PlaceSet::new(results.analysis.place_domain());
 
   let expanded_targets = targets
     .iter()
     .map(|(place, location)| {
-      let mut places = new_place_set();
-      places.insert(*place);
-
-      for (_, ptrs) in place.interior_pointers(tcx, body, results.analysis.def_id) {
-        for (place, _) in ptrs {
-          debug!(
-            "{:?} // {:?}",
-            tcx.mk_place_deref(place),
-            aliases.aliases.row_set(tcx.mk_place_deref(place))
-          );
-          places.union(&aliases.aliases.row_set(tcx.mk_place_deref(place)).unwrap());
-        }
-      }
+      let places = aliases.reachable_values(tcx, body, results.analysis.def_id, *place);
 
       (places, *location)
     })
     .collect::<Vec<_>>();
-  debug!(
-    "Expanded targets from {:?} to {:?}",
-    targets, expanded_targets
-  );
+  debug!("Expanded targets from {targets:?} to {expanded_targets:?}");
 
   let target_deps = {
     let get_deps = |(targets, location): &(PlaceSet<'tcx>, Location)| {
@@ -172,22 +163,21 @@ pub fn compute_dependencies(
 
       let mut locations = new_location_set();
       for target in targets.indices() {
-        if let Some(dep_locations) = state.row_set(target) {
-          locations.union(&dep_locations);
-        }
+        locations.union(&state.row_set(target));
       }
 
       locations
     };
     expanded_targets.iter().map(get_deps).collect::<Vec<_>>()
   };
-  debug!("Target deps: {:?}", target_deps);
+  debug!("Target deps: {target_deps:?}");
 
   let mut outputs = target_deps
     .iter()
     .map(|_| (new_location_set(), new_place_set()))
     .collect::<Vec<_>>();
-  for ((target_places, _), (_, places)) in expanded_targets.iter().zip(outputs.iter_mut()) {
+  for ((target_places, _), (_, places)) in expanded_targets.iter().zip(outputs.iter_mut())
+  {
     places.union(target_places);
   }
 
@@ -207,33 +197,31 @@ pub fn compute_dependency_spans(
   results: &FlowResults<'_, 'tcx>,
   targets: Vec<(Place<'tcx>, Location)>,
   direction: Direction,
-  spanner: &HirSpanner,
+  spanner: &Spanner,
 ) -> Vec<Vec<Span>> {
   let tcx = results.analysis.tcx;
   let body = results.analysis.body;
 
-  let source_map = tcx.sess.source_map();
   let deps = compute_dependencies(results, targets, direction);
 
   deps
     .into_iter()
     .map(|(locations, places)| {
-      let location_spans = locations
-        .iter()
-        .map(|location| location_to_spans(*location, body, spanner, source_map))
-        .flatten();
+      let location_spans = locations.iter().flat_map(|location| {
+        spanner.location_to_spans(*location, EnclosingHirSpans::OuterOnly)
+      });
 
       let place_spans = places
         .iter()
-        .filter(|place| **place != Place::return_place())
-        .map(|place| {
+        .filter_map(|place| {
           body.local_decls()[place.local]
             .source_info
             .span
-            .source_callsite()
-        });
+            .as_local(tcx)
+        })
+        .filter(|span| !span.source_equal(&spanner.body_span));
 
-      location_spans.chain(place_spans).collect::<Vec<_>>()
+      Span::merge_overlaps(location_spans.chain(place_spans).collect::<Vec<_>>())
     })
     .collect::<Vec<_>>()
 }
