@@ -1,21 +1,26 @@
-use crate::{
-  block_timer,
-  indexed::impls::LocationDomain,
-  mir::{aliases::Aliases, control_dependencies::ControlDependencies, engine, utils::BodyExt},
-};
-use log::debug;
-use rustc_borrowck::consumers::BodyWithBorrowckFacts;
-use rustc_hir::BodyId;
-use rustc_middle::ty::TyCtxt;
 use std::cell::RefCell;
 
 pub use analysis::{FlowAnalysis, FlowDomain};
 pub use dependencies::{compute_dependencies, compute_dependency_spans, Direction};
+use log::debug;
+use rustc_borrowck::consumers::BodyWithBorrowckFacts;
+use rustc_hir::BodyId;
+use rustc_middle::ty::TyCtxt;
+
+use crate::{
+  block_timer,
+  indexed::impls::LocationDomain,
+  mir::{
+    aliases::Aliases, control_dependencies::ControlDependencies, engine, utils::BodyExt,
+  },
+};
 
 mod analysis;
 mod dependencies;
+pub mod mutation;
+mod recursive;
 
-pub type FlowResults<'a, 'b> = engine::AnalysisResults<'b, FlowAnalysis<'a, 'b>>;
+pub type FlowResults<'a, 'tcx> = engine::AnalysisResults<'tcx, FlowAnalysis<'a, 'tcx>>;
 
 thread_local! {
   pub static BODY_STACK: RefCell<Vec<BodyId>> =
@@ -29,6 +34,11 @@ pub fn compute_flow<'a, 'tcx>(
 ) -> FlowResults<'a, 'tcx> {
   BODY_STACK.with(|body_stack| {
     body_stack.borrow_mut().push(body_id);
+    debug!(
+      "{}",
+      rustc_hir_pretty::to_string(rustc_hir_pretty::NO_ANN, |s| s
+        .print_expr(&tcx.hir().body(body_id).value))
+    );
     debug!("{}", body_with_facts.body.to_string(tcx).unwrap());
 
     let def_id = tcx.hir().body_owner_def_id(body_id).to_def_id();
@@ -36,7 +46,7 @@ pub fn compute_flow<'a, 'tcx>(
 
     let body = &body_with_facts.body;
     let control_dependencies = ControlDependencies::build(body.clone());
-    debug!("Control dependencies: {:?}", control_dependencies);
+    debug!("Control dependencies: {control_dependencies:?}");
 
     let location_domain = LocationDomain::new(body, &aliases.place_domain);
 
