@@ -1,9 +1,12 @@
 #![feature(
   rustc_private,             // for rustc internals
-  box_patterns,              // nice-to-have
-  in_band_lifetimes,         // nice-to-have
+  box_patterns,              // for conciseness
+  in_band_lifetimes,         // for conciseness
   associated_type_defaults,  // for crate::indexed::Indexed
   min_specialization,        // for rustc_index::newtype_index
+  type_alias_impl_trait,     // for impl Trait in trait definition, eg crate::mir::utils 
+  generic_associated_types,  // for impl Trait in trait definition
+  crate_visibility_modifier, // for crate-wide shared private items
 )]
 #![allow(
   clippy::single_match,
@@ -12,6 +15,8 @@
   clippy::len_zero
 )]
 
+extern crate datafrog;
+extern crate either;
 extern crate polonius_engine;
 extern crate rustc_borrowck;
 extern crate rustc_data_structures;
