@@ -1,5 +1,16 @@
-use crate::extensions::{is_extension_active, MutabilityMode};
+use std::{
+  cmp,
+  collections::hash_map::Entry,
+  hash::Hash,
+  io::Write,
+  iter,
+  ops::ControlFlow,
+  path::Path,
+  process::{Command, Stdio},
+};
+
 use anyhow::{bail, Result};
+use either::Either;
 use log::{trace, warn};
 use rustc_data_structures::fx::{FxHashMap as HashMap, FxHashSet as HashSet};
 use rustc_graphviz as dot;
@@ -8,23 +19,19 @@ use rustc_middle::{
   mir::{
     pretty::write_mir_fn,
     visit::{PlaceContext, Visitor},
-    *,
+    MirPass, *,
+  },
+  ty::{
+    self, AdtKind, RegionKind, RegionVid, Ty, TyCtxt, TyKind, TyS, TypeFoldable,
+    TypeVisitor,
   },
-  ty::{self, RegionKind, RegionVid, Ty, TyCtxt, TyKind, TyS, TypeFoldable, TypeVisitor},
 };
 use rustc_mir_dataflow::{fmt::DebugWithContext, graphviz, Analysis, Results};
-use rustc_mir_transform::MirPass;
-use rustc_span::Symbol;
+use rustc_span::{Pos, Span, Symbol, SyntaxContext};
 use rustc_target::abi::VariantIdx;
 use smallvec::SmallVec;
-use std::{
-  collections::hash_map::Entry,
-  hash::Hash,
-  io::Write,
-  ops::ControlFlow,
-  path::Path,
-  process::{Command, Stdio},
-};
+
+use crate::extensions::{is_extension_active, MutabilityMode};
 
 pub trait OperandExt<'tcx> {
   fn to_place(&self) -> Option<Place<'tcx>>;
@@ -45,14 +52,15 @@ pub fn arg_mut_ptrs<'tcx>(
   body: &Body<'tcx>,
   def_id: DefId,
 ) -> Vec<(usize, Place<'tcx>)> {
-  let ignore_mut = is_extension_active(|mode| mode.mutability_mode == MutabilityMode::IgnoreMut);
+  let ignore_mut =
+    is_extension_active(|mode| mode.mutability_mode == MutabilityMode::IgnoreMut);
   args
     .iter()
-    .map(|(i, place)| {
+    .flat_map(|(i, place)| {
       place
         .interior_pointers(tcx, body, def_id)
         .into_iter()
-        .map(|(_, places)| {
+        .flat_map(|(_, places)| {
           places
             .into_iter()
             .filter_map(|(place, mutability)| match mutability {
@@ -60,10 +68,8 @@ pub fn arg_mut_ptrs<'tcx>(
               Mutability::Not => ignore_mut.then(|| place),
             })
         })
-        .flatten()
         .map(move |place| (*i, tcx.mk_place_deref(place)))
     })
-    .flatten()
     .collect::<Vec<_>>()
 }
 
@@ -131,9 +137,9 @@ impl PlaceRelation {
 
     let is_sub_part = part_place.projection.len() >= whole_place.projection.len();
     let remaining_projection = if is_sub_part {
-      &part_place.projection[whole_place.projection.len()..]
+      &part_place.projection[whole_place.projection.len() ..]
     } else {
-      &whole_place.projection[part_place.projection.len()..]
+      &whole_place.projection[part_place.projection.len() ..]
     };
 
     if remaining_projection
@@ -155,15 +161,59 @@ impl PlaceRelation {
   }
 }
 
-#[derive(Default)]
 pub struct PlaceCollector<'tcx> {
-  pub places: Vec<Place<'tcx>>,
+  pub tcx: TyCtxt<'tcx>,
+  pub places: Vec<(Place<'tcx>, Option<PlaceElem<'tcx>>)>,
 }
 
 impl Visitor<'tcx> for PlaceCollector<'tcx> {
-  fn visit_place(&mut self, place: &Place<'tcx>, _context: PlaceContext, _location: Location) {
-    self.places.push(*place);
+  fn visit_place(
+    &mut self,
+    place: &Place<'tcx>,
+    _context: PlaceContext,
+    _location: Location,
+  ) {
+    self.places.push((*place, None));
   }
+
+  fn visit_rvalue(&mut self, rvalue: &Rvalue<'tcx>, location: Location) {
+    match rvalue {
+      Rvalue::Aggregate(box AggregateKind::Adt(def_id, idx, _substs, _, _), ops) => {
+        // In the case of _1 = aggregate { field1: op1, field2: op2, ... }
+        // we want to remember which places correspond to which fields so the infoflow
+        // analysis can be field-sensitive for constructors.
+        let adt_def = self.tcx.adt_def(*def_id);
+        let variant = &adt_def.variants[*idx];
+        let places =
+          (0 .. variant.fields.len())
+            .zip(ops.iter())
+            .filter_map(|(field, op)| {
+              let place = op.to_place()?;
+              let field =
+                ProjectionElem::Field(Field::from_usize(field), self.tcx.mk_unit());
+              Some((place, Some(field)))
+            });
+        self.places.extend(places);
+      }
+      _ => self.super_rvalue(rvalue, location),
+    }
+  }
+}
+
+pub fn run_dot(path: &Path, buf: Vec<u8>) -> Result<()> {
+  let mut p = Command::new("dot")
+    .args(&["-Tpdf", "-o", &path.display().to_string()])
+    .stdin(Stdio::piped())
+    .spawn()?;
+
+  p.stdin.as_mut().unwrap().write_all(&buf)?;
+  let status = p.wait()?;
+
+  if !status.success() {
+    bail!("dot for {} failed", path.display())
+  };
+
+  Ok(())
 }
 
 pub fn dump_results<'tcx, A>(
@@ -176,36 +226,23 @@ where
   A: Analysis<'tcx>,
   A::Domain: DebugWithContext<A>,
 {
-  let graphviz = graphviz::Formatter::new(body, results, graphviz::OutputStyle::AfterOnly);
+  let graphviz =
+    graphviz::Formatter::new(body, results, graphviz::OutputStyle::AfterOnly);
   let mut buf = Vec::new();
   dot::render(&graphviz, &mut buf)?;
 
   let output_dir = Path::new("target");
   // let fname = tcx.def_path_debug_str(def_id);
   let fname = "results";
-  let output_path = output_dir.join(format!("{}.png", fname));
+  let output_path = output_dir.join(format!("{fname}.png"));
 
-  let mut p = Command::new("dot")
-    .args(&["-Tpng", "-o", &output_path.display().to_string()])
-    .stdin(Stdio::piped())
-    .spawn()?;
-
-  p.stdin.as_mut().unwrap().write_all(&buf)?;
-  let status = p.wait()?;
-
-  if !status.success() {
-    bail!("dot for {} failed", output_path.display())
-  };
-
-  Ok(())
+  run_dot(&output_path, buf)
 }
 
 pub fn location_to_string(location: Location, body: &Body<'_>) -> String {
-  let block = &body.basic_blocks()[location.block];
-  if location.statement_index == block.statements.len() {
-    format!("{:?}", block.terminator().kind)
-  } else {
-    format!("{:?}", block.statements[location.statement_index].kind)
+  match body.stmt_at(location) {
+    Either::Left(stmt) => format!("{:?}", stmt.kind),
+    Either::Right(terminator) => format!("{:?}", terminator.kind),
   }
 }
 
@@ -216,8 +253,6 @@ impl MirPass<'tcx> for SimplifyMir {
       block.statements.retain(|stmt| {
         !matches!(
           stmt.kind,
-          // TODO: variable_select_lhs test fails if we remove FakeRead
-          // StatementKind::FakeRead(..)
           StatementKind::StorageLive(..) | StatementKind::StorageDead(..)
         )
       });
@@ -238,10 +273,13 @@ impl MirPass<'tcx> for SimplifyMir {
 
 pub trait PlaceExt<'tcx> {
   fn make(local: Local, projection: &[PlaceElem<'tcx>], tcx: TyCtxt<'tcx>) -> Self;
+  fn from_ref(place: PlaceRef<'tcx>, tcx: TyCtxt<'tcx>) -> Self;
   fn from_local(local: Local, tcx: TyCtxt<'tcx>) -> Self;
   fn is_arg(&self, body: &Body<'tcx>) -> bool;
   fn is_direct(&self, body: &Body<'tcx>) -> bool;
-  fn pointers_in_projection(&self, tcx: TyCtxt<'tcx>) -> SmallVec<[Place<'tcx>; 2]>;
+  fn refs_in_projection(&self) -> SmallVec<[(PlaceRef<'tcx>, &[PlaceElem<'tcx>]); 2]>;
+  fn place_and_refs_in_projection(&self, tcx: TyCtxt<'tcx>)
+    -> SmallVec<[Place<'tcx>; 2]>;
   fn interior_pointers(
     &self,
     tcx: TyCtxt<'tcx>,
@@ -255,6 +293,7 @@ pub trait PlaceExt<'tcx> {
     def_id: DefId,
     depth_limit: Option<usize>,
   ) -> Vec<Place<'tcx>>;
+  fn to_string(&self, tcx: TyCtxt<'tcx>, body: &Body<'tcx>) -> Option<String>;
 }
 
 impl PlaceExt<'tcx> for Place<'tcx> {
@@ -265,6 +304,10 @@ impl PlaceExt<'tcx> for Place<'tcx> {
     }
   }
 
+  fn from_ref(place: PlaceRef<'tcx>, tcx: TyCtxt<'tcx>) -> Self {
+    Self::make(place.local, place.projection, tcx)
+  }
+
   fn from_local(local: Local, tcx: TyCtxt<'tcx>) -> Self {
     Place::make(local, &[], tcx)
   }
@@ -278,16 +321,39 @@ impl PlaceExt<'tcx> for Place<'tcx> {
     !self.is_indirect() || self.is_arg(body)
   }
 
-  fn pointers_in_projection(&self, tcx: TyCtxt<'tcx>) -> SmallVec<[Place<'tcx>; 2]> {
+  fn refs_in_projection(&self) -> SmallVec<[(PlaceRef<'tcx>, &[PlaceElem<'tcx>]); 2]> {
     self
-      .iter_projections()
-      .filter_map(|(place_ref, elem)| match elem {
-        ProjectionElem::Deref => Some(Place::make(place_ref.local, place_ref.projection, tcx)),
+      .projection
+      .iter()
+      .enumerate()
+      .filter_map(|(i, elem)| match elem {
+        ProjectionElem::Deref => {
+          let ptr = PlaceRef {
+            local: self.local,
+            projection: &self.projection[.. i],
+          };
+          let after = &self.projection[i + 1 ..];
+          Some((ptr, after))
+        }
         _ => None,
       })
       .collect()
   }
 
+  fn place_and_refs_in_projection(
+    &self,
+    tcx: TyCtxt<'tcx>,
+  ) -> SmallVec<[Place<'tcx>; 2]> {
+    iter::once(*self)
+      .chain(
+        self
+          .refs_in_projection()
+          .into_iter()
+          .map(|(ptr, _)| Place::from_ref(ptr, tcx)),
+      )
+      .collect()
+  }
+
   fn interior_pointers(
     &self,
     tcx: TyCtxt<'tcx>,
@@ -332,6 +398,57 @@ impl PlaceExt<'tcx> for Place<'tcx> {
     region_collector.visit_ty(ty);
     region_collector.places.unwrap().into_iter().collect()
   }
+
+  fn to_string(&self, tcx: TyCtxt<'tcx>, body: &Body<'tcx>) -> Option<String> {
+    let local_name = if self.local == RETURN_PLACE {
+      "RETURN".into()
+    } else {
+      body
+        .var_debug_info
+        .iter()
+        .filter_map(|info| match info.value {
+          VarDebugInfoContents::Place(place) if place.local == self.local => info
+            .source_info
+            .span
+            .as_local(tcx)
+            .map(|_| info.name.to_string()),
+          _ => None,
+        })
+        .next()?
+    };
+
+    let projection_to_string =
+      |s: String, (place, elem): (PlaceRef<'tcx>, PlaceElem<'tcx>)| match elem {
+        ProjectionElem::Deref => format!("*{s}"),
+        ProjectionElem::Field(f, _) => {
+          let ty = place.ty(&body.local_decls, tcx).ty;
+          let default = || format!("{s}.{}", f.as_usize());
+          if let Some(def) = ty.ty_adt_def() {
+            match def.adt_kind() {
+              AdtKind::Struct => {
+                let name = def.non_enum_variant().fields[f.as_usize()].ident(tcx);
+                format!("{s}.{name}")
+              }
+              _ => default(),
+            }
+          } else {
+            default()
+          }
+        }
+        ProjectionElem::Downcast(sym, _) => format!(
+          "{s} as {}",
+          sym.map(|s| s.to_string()).unwrap_or_else(|| "??".into())
+        ),
+        ProjectionElem::Index(_) => format!("{s}[]"),
+        _ => unimplemented!(),
+      };
+
+    Some(
+      self
+        .iter_projections()
+        .fold(local_name, projection_to_string),
+    )
+  }
 }
 
 struct CollectRegions<'tcx> {
@@ -347,10 +464,6 @@ struct CollectRegions<'tcx> {
 }
 
 impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
-  fn tcx_for_anon_const_substs(&self) -> Option<TyCtxt<'tcx>> {
-    Some(self.tcx)
-  }
-
   fn visit_ty(&mut self, ty: Ty<'tcx>) -> ControlFlow<Self::BreakTy> {
     if self
       .ty_stack
@@ -367,9 +480,8 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
     }
 
     trace!(
-      "exploring {:?} with {:?}",
-      Place::make(self.local, &self.place_stack, self.tcx),
-      ty
+      "exploring {:?} with {ty:?}",
+      Place::make(self.local, &self.place_stack, self.tcx)
     );
 
     self.ty_stack.push(ty);
@@ -414,7 +526,7 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
           for (i, variant) in adt_def.variants.iter().enumerate() {
             let variant_index = VariantIdx::from_usize(i);
             let cast = PlaceElem::Downcast(
-              Some(adt_def.variants[variant_index].ident.name),
+              Some(adt_def.variants[variant_index].ident(self.tcx).name),
               variant_index,
             );
             self.place_stack.push(cast);
@@ -462,7 +574,7 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
       _ if ty.is_primitive_ty() => {}
 
       _ => {
-        warn!("unimplemented {:?} ({:?})", ty, ty.kind());
+        warn!("unimplemented {ty:?} ({:?})", ty.kind());
       }
     };
 
@@ -479,7 +591,7 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
   }
 
   fn visit_region(&mut self, region: ty::Region<'tcx>) -> ControlFlow<Self::BreakTy> {
-    trace!("visiting region {:?}", region);
+    trace!("visiting region {region:?}");
     match region {
       RegionKind::ReVar(region) => {
         let mutability = if self
@@ -514,13 +626,30 @@ impl TypeVisitor<'tcx> for CollectRegions<'tcx> {
 }
 
 pub trait BodyExt<'tcx> {
-  fn all_returns(&self) -> Vec<Location>;
+  type AllReturnsIter<'a>: Iterator<Item = Location>
+  where
+    Self: 'a;
+  fn all_returns(&self) -> Self::AllReturnsIter<'_>;
+
+  type AllLocationsIter<'a>: Iterator<Item = Location>
+  where
+    Self: 'a;
+  fn all_locations(&self) -> Self::AllLocationsIter<'_>;
+
+  type LocationsIter: Iterator<Item = Location>;
+  fn locations_in_block(&self, block: BasicBlock) -> Self::LocationsIter;
+
   fn debug_info_name_map(&self) -> HashMap<Local, Symbol>;
+
   fn to_string(&self, tcx: TyCtxt<'tcx>) -> Result<String>;
 }
 
 impl BodyExt<'tcx> for Body<'tcx> {
-  fn all_returns(&self) -> Vec<Location> {
+  type AllReturnsIter<'a>
+  where
+    Self: 'a,
+  = impl Iterator<Item = Location>;
+  fn all_returns(&self) -> Self::AllReturnsIter<'_> {
     self
       .basic_blocks()
       .iter_enumerated()
@@ -531,7 +660,31 @@ impl BodyExt<'tcx> for Body<'tcx> {
         }),
         _ => None,
       })
-      .collect()
+  }
+
+  type AllLocationsIter<'a>
+  where
+    Self: 'a,
+  = impl Iterator<Item = Location>;
+  fn all_locations(&self) -> Self::AllLocationsIter<'_> {
+    self
+      .basic_blocks()
+      .iter_enumerated()
+      .flat_map(|(block, data)| {
+        (0 .. data.statements.len() + 1).map(move |statement_index| Location {
+          block,
+          statement_index,
+        })
+      })
+  }
+
+  type LocationsIter = impl Iterator<Item = Location>;
+  fn locations_in_block(&self, block: BasicBlock) -> Self::LocationsIter {
+    let num_stmts = self.basic_blocks()[block].statements.len();
+    (0 ..= num_stmts).map(move |statement_index| Location {
+      block,
+      statement_index,
+    })
   }
 
   fn debug_info_name_map(&self) -> HashMap<Local, Symbol> {
@@ -551,3 +704,194 @@ impl BodyExt<'tcx> for Body<'tcx> {
     Ok(String::from_utf8(buffer)?)
   }
 }
+
+pub trait SpanExt {
+  fn subtract(&self, child_spans: Vec<Span>) -> Vec<Span>;
+  fn as_local(&self, tcx: TyCtxt<'tcx>) -> Option<Span>;
+  fn overlaps_inclusive(&self, other: Span) -> bool;
+  fn trim_end(&self, other: Span) -> Option<Span>;
+  fn merge_overlaps(spans: Vec<Span>) -> Vec<Span>;
+  fn to_string(&self, tcx: TyCtxt<'_>) -> String;
+  fn size(&self) -> u32;
+}
+
+impl SpanExt for Span {
+  fn trim_end(&self, other: Span) -> Option<Span> {
+    let span = self.data();
+    let other = other.data();
+    if span.lo < other.lo {
+      Some(span.with_hi(cmp::min(span.hi, other.lo)))
+    } else {
+      None
+    }
+  }
+
+  /// Get spans for regions in `self` not in `child_spans`.
+  ///
+  /// Example:
+  ///  self:          ---------------
+  ///  child_spans:    ---      --  -
+  ///  output:        -   ------  --
+  fn subtract(&self, mut child_spans: Vec<Span>) -> Vec<Span> {
+    let mut outer_spans = vec![];
+    if !child_spans.is_empty() {
+      child_spans.retain(|s| s.overlaps_inclusive(*self));
+
+      // Output will be sorted
+      child_spans = Span::merge_overlaps(child_spans);
+
+      if let Some(start) = self.trim_end(*child_spans.first().unwrap()) {
+        outer_spans.push(start);
+      }
+
+      for children in child_spans.windows(2) {
+        outer_spans.push(children[0].between(children[1]));
+      }
+
+      if let Some(end) = self.trim_start(*child_spans.last().unwrap()) {
+        outer_spans.push(end);
+      }
+    } else {
+      outer_spans.push(*self);
+    };
+
+    trace!("outer span for {self:?} with inner spans {child_spans:?} is {outer_spans:?}");
+
+    outer_spans
+  }
+
+  fn as_local(&self, tcx: TyCtxt<'_>) -> Option<Span> {
+    // Before we call source_callsite, we check and see if the span is already local.
+    // This is important b/c in print!("{}", y) if the user selects `y`, the source_callsite
+    // of that span is the entire macro.
+    let is_local = |sp| tcx.sess.source_map().is_local_span(sp);
+    if is_local(*self) {
+      return Some(*self);
+    } else {
+      let sp = self.source_callsite();
+      if is_local(sp) {
+        return Some(sp);
+      }
+    }
+
+    None
+  }
+
+  fn overlaps_inclusive(&self, other: Span) -> bool {
+    let s1 = self.data();
+    let s2 = other.data();
+    s1.lo <= s2.hi && s2.lo <= s1.hi
+  }
+
+  fn merge_overlaps(mut spans: Vec<Span>) -> Vec<Span> {
+    spans.sort_by_key(|s| (s.lo(), s.hi()));
+
+    // See note in Span::subtract
+    for span in spans.iter_mut() {
+      *span = span.with_ctxt(SyntaxContext::root());
+    }
+
+    let mut output = Vec::new();
+    for span in spans {
+      match output
+        .iter_mut()
+        .find(|other| span.overlaps_inclusive(**other))
+      {
+        Some(other) => {
+          *other = span.to(*other);
+        }
+        None => {
+          output.push(span);
+        }
+      }
+    }
+    output
+  }
+
+  fn to_string(&self, tcx: TyCtxt<'_>) -> String {
+    let source_map = tcx.sess.source_map();
+    let lo = source_map.lookup_char_pos(self.lo());
+    let hi = source_map.lookup_char_pos(self.hi());
+    let snippet = source_map.span_to_snippet(*self).unwrap();
+    format!(
+      "{snippet} ({}:{}-{}:{})",
+      lo.line,
+      lo.col.to_usize() + 1,
+      hi.line,
+      hi.col.to_usize() + 1
+    )
+  }
+
+  fn size(&self) -> u32 {
+    self.hi().0 - self.lo().0
+  }
+}
+
+// #[derive(Debug)]
+// struct Loop {
+//   header: Location,
+//   body: LocationSet,
+// }
+
+// fn find_loops(
+//   body: &Body,
+//   location_domain: &Rc<LocationDomain>,
+// ) -> (Vec<Loop>, LocationSet) {
+//   let mut loops = vec![];
+//   for node in body.basic_blocks().indices() {
+//     for successor in body.successors(node) {
+//       if body.dominators().is_dominated_by(node, successor) {
+//         let mut loop_body = HashSet::default();
+//         loop_body.insert(successor);
+
+//         let mut stack = vec![node];
+//         while !stack.is_empty() {
+//           let n = stack.pop().unwrap();
+//           loop_body.insert(n);
+//           stack.extend(
+//             body.predecessors()[n]
+//               .iter()
+//               .filter(|p| **p != successor && !loop_body.contains(p))
+//               .copied(),
+//           );
+//         }
+
+//         loops.push(Loop {
+//           header: successor.start_location(),
+//           body: loop_body
+//             .into_iter()
+//             .map(|block| body.locations_in_block(block))
+//             .flatten()
+//             .collect_indices(location_domain.clone()),
+//         });
+//       }
+//     }
+//   }
+
+//   let mut outer = body
+//     .all_locations()
+//     .collect_indices(location_domain.clone());
+//   for l in &loops {
+//     outer.subtract(&l.body);
+//   }
+
+//   (loops, outer)
+// }
+
+#[cfg(test)]
+mod test {
+  use rustc_span::BytePos;
+
+  use super::*;
+
+  #[test]
+  fn test_span_subtract() {
+    rustc_span::create_default_session_if_not_set_then(|_| {
+      let mk = |lo, hi| Span::with_root_ctxt(BytePos(lo), BytePos(hi));
+      let outer = mk(1, 10);
+      let inner: Vec<Span> = vec![mk(0, 2), mk(3, 4), mk(3, 5), mk(7, 8), mk(9, 13)];
+      let desired: Vec<Span> = vec![mk(2, 3), mk(5, 7), mk(8, 9)];
+      assert_eq!(outer.subtract(inner), desired);
+    });
+  }
+}
