@@ -1,13 +1,14 @@
-use crate::mir::borrowck_facts;
+use std::{io, path::Path, process::Command};
+
 use anyhow::{anyhow, bail, Context, Result};
+use log::debug;
+use rustc_borrowck::BodyWithBorrowckFacts;
 use rustc_data_structures::fx::FxHashMap as HashMap;
 use rustc_hir::{BodyId, ItemKind};
-use rustc_middle::{
-  mir::Body,
-  ty::{TyCtxt, WithOptConstParam},
-};
+use rustc_middle::ty::TyCtxt;
 use rustc_span::{source_map::FileLoader, BytePos, Span, SyntaxContext};
-use std::{io, path::Path, process::Command};
+
+use crate::mir::{borrowck_facts, utils::BodyExt};
 
 struct StringLoader(String);
 impl FileLoader for StringLoader {
@@ -34,7 +35,7 @@ lazy_static::lazy_static! {
 
 pub fn compile_body(
   input: impl Into<String>,
-  callback: impl for<'tcx> FnOnce(TyCtxt<'tcx>, BodyId, &Body<'tcx>) + Send,
+  callback: impl for<'tcx> FnOnce(TyCtxt<'tcx>, BodyId, &BodyWithBorrowckFacts<'tcx>) + Send,
 ) {
   compile(input, |tcx| {
     let body_id = tcx
@@ -48,9 +49,10 @@ pub fn compile_body(
       .unwrap();
 
     let def_id = tcx.hir().body_owner_def_id(body_id);
-    let body = tcx.mir_built(WithOptConstParam::unknown(def_id)).borrow();
+    let body_with_facts = borrowck_facts::get_body_with_borrowck_facts(tcx, def_id);
+    debug!("{}", body_with_facts.body.to_string(tcx).unwrap());
 
-    callback(tcx, body_id, &*body);
+    callback(tcx, body_id, body_with_facts);
   })
 }
 
@@ -58,7 +60,10 @@ pub fn compile(input: impl Into<String>, callback: impl FnOnce(TyCtxt<'_>) + Sen
   let mut callbacks = TestCallbacks {
     callback: Some(callback),
   };
-  let args = format!("rustc dummy.rs --crate-type lib --sysroot {}", &*SYSROOT);
+  let args = format!(
+    "rustc dummy.rs --crate-type lib -Z identify-regions -Z mir-opt-level=0 --sysroot {}",
+    &*SYSROOT
+  );
   let args = args.split(' ').map(|s| s.to_string()).collect::<Vec<_>>();
 
   rustc_driver::catch_fatal_errors(|| {
@@ -95,10 +100,12 @@ where
   }
 }
 
+pub type RangeMap = HashMap<&'static str, Vec<(usize, usize)>>;
+
 pub fn parse_ranges(
   src: impl AsRef<str>,
   delimiters: impl AsRef<[(&'static str, &'static str)]>,
-) -> Result<(String, HashMap<&'static str, Vec<(usize, usize)>>)> {
+) -> Result<(String, RangeMap)> {
   let src = src.as_ref();
   let delimiters = delimiters.as_ref();
 
@@ -116,7 +123,8 @@ pub fn parse_ranges(
       $tokens
         .iter()
         .find(|t| {
-          in_idx + t.len() <= bytes.len() && t.as_bytes() == &bytes[in_idx..in_idx + t.len()]
+          in_idx + t.len() <= bytes.len()
+            && t.as_bytes() == &bytes[in_idx .. in_idx + t.len()]
         })
         .map(|t| *t)
     };
@@ -132,7 +140,7 @@ pub fn parse_ranges(
     if let Some(close) = check_token!(&closes) {
       let (start, delim) = stack
         .pop()
-        .with_context(|| anyhow!("Missing open delimiter for \"{}\"", close))?;
+        .with_context(|| anyhow!("Missing open delimiter for \"{close}\""))?;
       ranges.entry(delim).or_default().push((start, out_idx));
       in_idx += close.len();
       continue;
@@ -144,7 +152,7 @@ pub fn parse_ranges(
   }
 
   if stack.len() > 0 {
-    bail!("Unclosed delimiters: {:?}", stack);
+    bail!("Unclosed delimiters: {stack:?}");
   }
 
   let prog_clean = String::from_utf8(buf)?;
@@ -159,3 +167,16 @@ pub fn make_span((lo, hi): (usize, usize)) -> Span {
     None,
   )
 }
+
+#[cfg(test)]
+mod test {
+  use super::*;
+
+  #[test]
+  fn test_parse_ranges() {
+    let s = "`[`[f]`oo]`";
+    let (clean, ranges) = parse_ranges(s, vec![("`[", "]`")]).unwrap();
+    assert_eq!(clean, "foo");
+    assert_eq!(ranges["`["], vec![(0, 1), (0, 3)])
+  }
+}
