@@ -1,8 +1,9 @@
-use log::info;
 use std::time::Instant;
 
+use log::info;
+
 pub fn elapsed(name: &str, start: Instant) {
-  info!("{} took {}s", name, start.elapsed().as_secs_f64());
+  info!("{name} took {}s", start.elapsed().as_secs_f64());
 }
 
 pub struct BlockTimer<'a> {
